@@ -225,7 +225,7 @@ impl NoteIndex {
             .unwrap_or_default()
     }
 
-    /// Returns an iterator over pairs of (id, name) of notes linking to this note.
+    /// Returns a vector over pairs of (id, name) of notes linking to this note.
     pub fn blinks_vec(&self, target_id: &str) -> Vec<(String, String)> {
         let id_copy = target_id.to_string();
         self.inner
@@ -237,6 +237,21 @@ impl NoteIndex {
             .collect()
     }
 
+    /// Returns a map of all tags contained in this index with their number of appearances, sorted in descending order.
+    pub fn tags_vec(&self) -> Vec<(String, usize)> {
+        self.inner
+            .iter()
+            .flat_map(|(_id, note)| note.tags.iter())
+            .cloned()
+            .counts()
+            .into_iter()
+            .sorted_by_key(|(tag, _count)| tag.clone())
+            .rev()
+            .sorted_by_key(|(_tag, count)| *count)
+            .rev()
+            .collect()
+    }
+
     /// Requests this index to update itself to be in sync with the tracked file system.
     pub fn poll_file_system(&self) {
         self.tracker.poll_file_system();
@@ -267,6 +282,7 @@ impl NoteIndex {
 
 #[cfg(test)]
 mod tests {
+    use pretty_assertions::assert_eq;
     use super::*;
     use crate::io;
 
